//==========================================================================
//              DiffSolve.C  v. 1.0
//
//      Anharmonischer Oszillator als Sound-Generator.
//
//      written by Peter Schmitteckert
//      email: peter@schmitteckert.com
//
//      This source may be distributed, changed or deleted without any restriction.
//      Comments are welcome
//
//==========================================================================

#include <math.h>

#include "DiffSolve.h"

double CAnhOszi::DiffEquation_1( const double x0, const double x1) const
{
  return x1;
}

double CAnhOszi::DiffEquation_2( const double x0, const double x1) const
{
 return  a*x1 - b*x0 - c1*x0*x0*x0 - c2/(1.0+x0*x0) - c3*fmod( x0, 0.5);
 //return  a*x1 - b*x0 - c1*x0*x0*x0 - c2/(1+x0) - c3*sin(x0);
}

double CAnhTOszi::DiffEquation_1( const double t, const double x0, const double x1) const
{
  return x1;
}

double CAnhTOszi::DiffEquation_2( const double t, const double x0, const double x1) const
{
 return  a*x1 - b*x0 - c1*x0*x0*x0 - c2*cos( c3*t);
}


void StepRunge( const CLinDiffEq2* pLinDiffEq, double& x0, double& x1, const double StepSize)
{
	const int OverSampling = 5;

	const double h = StepSize / double( OverSampling );
	const double h2 = 0.5*h;

	for( int i=0; i<OverSampling; ++i)
	{
		const double k1 = pLinDiffEq->DiffEquation_1( x0, x1 );
		const double l1 = pLinDiffEq->DiffEquation_2( x0, x1 );

		const double k2 = pLinDiffEq->DiffEquation_1( (x0 + h2*k1), (x1 + h2*l1) );
		const double l2 = pLinDiffEq->DiffEquation_2( (x0 + h2*k1), (x1 + h2*l1) );

		const double k3 = pLinDiffEq->DiffEquation_1( (x0 + h2*k2), (x1 + h2*l2) );
		const double l3 = pLinDiffEq->DiffEquation_2( (x0 + h2*k2), (x1 + h2*l2) );

		const double k4 = pLinDiffEq->DiffEquation_1( (x0 + h *k3), (x1 + h *l3) );
		const double l4 = pLinDiffEq->DiffEquation_2( (x0 + h *k3), (x1 + h *l3) );

		const double h6 = h/6.0;
		x0 += h6*( k1 + k2+k2 + k3+k3 + k4);
		x1 += h6*( l1 + l2+l2 + l3+l3 + l4);
	}
}

// Runge-Kutta 4.ter Ordnung ohne StepSize Kontrolle fr zeitabhngige Probleme
void StepRunge( const CLinDiffEq2t* pLinDiffEq, double &t, double& x0, double& x1, const double StepSize)
{
	const int OverSampling = 5;

	const double h = StepSize / double( OverSampling );
	const double h2 = 0.5*h;

	for( int i=0; i<OverSampling; ++i)
	{
		const double k1 = pLinDiffEq->DiffEquation_1( t, x0, x1 );
		const double l1 = pLinDiffEq->DiffEquation_2( t, x0, x1 );

		const double k2 = pLinDiffEq->DiffEquation_1( (t + h2), (x0 + h2*k1), (x1 + h2*l1) );
		const double l2 = pLinDiffEq->DiffEquation_2( (t + h2), (x0 + h2*k1), (x1 + h2*l1) );

		const double k3 = pLinDiffEq->DiffEquation_1( (t + h2), (x0 + h2*k2), (x1 + h2*l2) );
		const double l3 = pLinDiffEq->DiffEquation_2( (t + h2), (x0 + h2*k2), (x1 + h2*l2) );

		t += h;
		const double k4 = pLinDiffEq->DiffEquation_1( t, (x0 + h *k3), (x1 + h *l3) );
		const double l4 = pLinDiffEq->DiffEquation_2( t, (x0 + h *k3), (x1 + h *l3) );

		const double h6 = h/6.0;
		x0 += h6*( k1 + k2+k2 + k3+k3 + k4);
		x1 += h6*( l1 + l2+l2 + l3+l3 + l4);
	}
}