//==========================================================================
//              DiffSolve.h  v. 1.0
//
//      Anharmonischer Oszillator als Sound-Generator.
//
//      written by Peter Schmitteckert
//      email: peter@schmitteckert.com
//
//      This source may be distributed, changed or deleted without any restriction.
//      Comments are welcome
//
//==========================================================================

#ifndef INCLUDES_DiffSolve
#define INCLUDES_DiffSolve



class CLinDiffEq2
// 2 gekoppelte Diffgleichungen 1. Ordner, ohne explizite Zeitbhngigkeit
{

 public:
	virtual double DiffEquation_1( const double x0, const double x1 ) const=0;
	virtual double DiffEquation_2( const double x0, const double x1 ) const=0;
};


class CLinDiffEq2t
// 2 gekoppelte Diffgleichungen 1. Ordner, mit explizite Zeitbhngigkeit
{

 public:
	virtual double DiffEquation_1( const double t, const double x0, const double x1 ) const=0;
	virtual double DiffEquation_2( const double t, const double x0, const double x1 ) const=0;
};

void StepRunge( const CLinDiffEq2* pLinDiffEq, double& x0, double& x1, const double h);
void StepRunge( const CLinDiffEq2t* pLinDiffEq, double &t, double& x0, double& x1, const double StepSize);

//-----------------------------

class CAnhOszi : public CLinDiffEq2
{
 //-----------------------------------------------
 // x'' + a*x' + b*x0 + c*x0*x0*x0 = 0
 //
 // -->
 //
 // x0' = x1
 // x1' = - a*x1 -b*x0 - c*x0*x0*x0
 //-----------------------------------------------

 public:
 	double a; // Reibungterm
	double b; // Oszillator Term, b =omega^2
	double c1,c2,c3; // Anharmonische Terme

	double DiffEquation_1( const double x0, const double x1) const;
	double DiffEquation_2( const double x0, const double x1) const;

};

class CAnhTOszi : public CLinDiffEq2t
{
 //-----------------------------------------------
 // x'' + a*x' + b*x0 + c*x0*x0*x0 = c2 * cos( c3*t)
 //
 // -->
 //
 // x0' = x1
 // x1' = - a*x1 -b*x0 - c*x0*x0*x0
 //-----------------------------------------------

 public:
 	double a; // Reibungterm
	double b; // Oszillator Term, b =omega^2
	double c1,c2,c3; // Anharmonische Terme

	double DiffEquation_1( const double t, const double x0, const double x1) const;
	double DiffEquation_2( const double t, const double x0, const double x1) const;

};

#endif // INCLUDES_DiffSolve
