//==========================================================================
//              DiffSound.C  v. 1.0
//
//      Anharmonischer Oszillator als Sound-Generator.
//
//      written by Peter Schmitteckert
//      email: peter@schmitteckert.com
//
//      This source may be distributed, changed or deleted without any restriction.
//      Comments are welcome
//
//==========================================================================

// ----------------------------------------------
// Samples:
// DiffSound 0.0 440 0 10000000 20000000     # Jmmerliches Gejohle
// DiffSound 0.0 -440 1000000 25000 1000000  # Bauchschmerzen
// ----------------------------------------------

#include <sys/soundcard.h>
#include <fcntl.h>
#include <unistd.h>

#include <sys/time.h>
#include <sys/ioctl.h>
#include <math.h>

#include <stdlib.h>
#include <stdio.h>
#include "DiffSolve.h"

const int BuffSize =  8*1024;

const char AudioDeviceName[] = "/dev/dsp" ;
const int AudioFormat =  AFMT_S16_NE;
typedef signed short TpAudioFormat;

int main( int argc, char* argv[] )
{
	int fdAudio;

	TpAudioFormat Buffer[BuffSize];
 	int len;
	int nFormat;
	int nChannels;
	int nSamplingRate;
	audio_buf_info info;

 	if(  (fdAudio = open( AudioDeviceName, O_WRONLY, 0)) == -1 )
	{
    		perror( AudioDeviceName );
		exit(-1);
	}

	// Setze Audio-Format: 16Bit signed integer, verwende host endian
	nFormat = AudioFormat;
	if( ioctl( fdAudio, SNDCTL_DSP_SETFMT, &nFormat ) == -1 )
	{
		perror("SNDCTL_DSP_SETFMT");
		exit(-2);
	}
	if( nFormat != AudioFormat)
	{
		perror("Versuchen Sie ein anderes AudioFormat");
		exit(-3);
	}

	// Setze Mono: 1 channel
	nChannels = 1;
	if( ioctl( fdAudio, SNDCTL_DSP_CHANNELS, &nChannels ) == -1 )
	{
		perror("SNDCTL_DSP_SETFMT");
		exit(-4);
	}
	if( nChannels != 1)
	{
		perror("Das Device untertzt kein Mono-Mode.");
		exit(-5);
	}

	// Setze Mono: 1 channel
	nSamplingRate = 44100;
	if( ioctl( fdAudio, SNDCTL_DSP_SPEED, &nSamplingRate ) == -1 )
	{
		perror("Sampling Rate");
		exit(-6);
	}
	printf("Current samping rate: %6d\n", nSamplingRate);

	if( ioctl( fdAudio, SNDCTL_DSP_GETBLKSIZE, &len ) == -1 )
	{
		perror("SNDCTL_DSP_GETBLKSIZE");
		exit(-6);
	}
	printf("Fragment Size: %x \t%d\n", len, len);


 	if( ioctl( fdAudio, SNDCTL_DSP_GETOSPACE, &info ) == -1 )
	{
		perror("SNDCTL_DSP_GETBLKSIZE");
		exit(-6);
	}
	printf("Fragments Size: %d \t%d \t%d \t%d \tBuffSize %d\n", info.fragments, info.fragstotal, info.fragsize, info.bytes, BuffSize);

	// --------------------------------------------------------------------------

	//CAnhTOszi AnhOszi;
	CAnhOszi AnhOszi;

	const double f = -440;
	const double Pi = 3.141992635;

	const double omega = (2.0*Pi) * f ;
	AnhOszi.a = 0.3;// * omega;
	AnhOszi.b = omega*fabs(omega);
	AnhOszi.c1 = 1000000;
	AnhOszi.c2 =  500000;
	AnhOszi.c3 = 0;

	if( argc > 1)
		AnhOszi.a = atof( argv[1] );

	if( argc > 2)
	{
		AnhOszi.b = (2.0*Pi) * atof( argv[2] );
		AnhOszi.b *= fabs(AnhOszi.b);
	}


	if( argc > 3)
		AnhOszi.c1 = atof( argv[3] );

	if( argc > 4)
		AnhOszi.c2 = atof( argv[4] );

	if( argc > 5)
		AnhOszi.c3 = atof( argv[5] );

	double x;
	double x0 = 0.1;
	double x1 = -0.01;
	double h = 1.0 / double(nSamplingRate);
	double t = 0;
	double max;
	while(1)
	{
		max = 0;

		for( int i=0; i<BuffSize; ++i, t+=h)
		{
			//StepRunge( &AnhOszi, t, x0, x1, h );
			StepRunge( &AnhOszi, x0, x1, h );
			//x = tanh( x0); // "Sanftes" beschrnken auf -1...+1
			x = 0.1 * x0;
			if( fabs(x) > max)
				max = fabs(x);

			Buffer[i] = (TpAudioFormat)(x * 32000. );
  		}
		printf("Max: %10.8lf\n", max);
		if( (len =  write( fdAudio,  Buffer,  2*BuffSize)) == -1)
   		{
  			perror("Audio-Ausgabe gescheitert.");
   	  		exit(-2);
		}
	}

 	printf("Ende\n");


  return 0;
}